import debounce from 'lodash/debounce';
import validate from '@meta-awesome/validators';
import validateRequired from '@meta-awesome/validators/src/required';

export default {
  data() {
    return {
      registred: false,
      originalRules: [],
      validator: {
        touched: false,
        isValid: true,
        error: false,
        errorText: ''
      }
    };
  },

  props: {
    /**
     * String containing the custom error
     * */
    error: {
      type: String,
      required: false,
      default: null
    },
    /**
     * Array containing validator functions
     * */
    rules: {
      type: Array,
      required: false,
      default: () => []
    }
  },

  methods: {
    validateValue(value = this.value) {
      const response = validate(value, ...this.computedRules);
      this.validator = {
        ...this.validator,
        ...response,
        error: !response.isValid && this.validator.touched
      };

      this.$emit('validation', this.validator, this.name);
      return this.validator;
    },

    setTouched(value) {
      if (!this.validator.touched) {
        this.validator = {
          ...this.validator,
          touched: true
        };

        this.validateValue(value);
      }
    },

    onlySetTouched() {
      if (!this.validator.touched) {
        this.validator = {
          ...this.validator,
          touched: true
        };
      }
    }
  },

  computed: {
    computedRules() {
      let rules = [...this.originalRules, ...this.rules];

      if (this.required) {
        rules = [...rules, validateRequired];
      }

      return rules;
    },
    hasError() {
      return !!this.error || this.validator.error;
    },
    errorText() {
      return this.error || (this.validator.error && this.validator.errorText);
    }
  },

  inject: ['form'],

  mounted() {
    if (this.form) {
      this.form.$emit('registerField', this);
    }
  },

  destroyed() {
    if (this.form) {
      this.form.$emit('unregisterField', this);
    }
  }
};
